#include <stdint.h>
#include <stdio.h>
#include <string.h> 
#include <math.h>
#include <stdlib.h>
#include <util/delay.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include <avr/iocan128.h>
#include <avr/iocanxx.h>
#include <avr/eeprom.h>
#include <avr/wdt.h>
#include "can.h"
#define TIMEOUT_C 5000 /*Timeout for CAN-communication*/


void Can_transmit1 (struct MOb1 *msg1)

{
	 
	CANPAGE = (0<<MOBNB0);/*set channel number */	
	CANSTMOB = 0x00;    	/* cancel pending operation */
	CANCDMOB = 0x00;
	CANIDT4 = (1<<RTRTAG);          // RTR=0 Frame with Data; RBO= no reservation
	CANCDMOB = ((msg1->length1)<<DLC0);   //Set lenght of Data
	// set Identifier
	CANIDT2 = ((msg1->id1)&0x7)<<5;                //set Identifier to send 
	CANIDT1 = (msg1->id1)>>3;                      //set Identifier to send
	CANIDM1 = 0xFF;
	CANIDM2 = 0xFF;
	CANIDM3 = 0xFF;
	CANIDM4 = 0xFF;
	for (unsigned int i = 0; i < 8; i++)
	{
		CANMSG = msg1->data1[i];    //Put Data in Mailbox
	
	}
	CANSTMOB = 0;
	
	CANCDMOB |= (1<<CONMOB0); /*enable transmition mode*/
	Wait_for_Can_Receive_message_Finished();
		
}

 
 
void Can_transmit2 (struct MOb2 *msg2)

{
	 
	CANPAGE = (0<<MOBNB0);/*set channel number */
	
	CANSTMOB = 0x00;    	/* cancel pending operation */
	CANCDMOB = 0x00;
	CANIDT4 = (1<<RTRTAG);          // RTR=0 Frame with Data; RBO= no reservation
	CANCDMOB = ((msg2->length2)<<DLC0);   //Set lenght of Data
        // set remote request

	// set Identifier
	CANIDT2 = ((msg2->id2)&0x7)<<5;                //set Identifier to send 
	CANIDT1 = (msg2->id2)>>3;                      //set Identifier to send
	CANIDM1 = 0xFF;
	CANIDM2 = 0xFF;
	CANIDM3 = 0xFF;
	CANIDM4 = 0xFF;
	for (unsigned int i = 0; i < 8; i++)
	{
		CANMSG = msg2->data2[i];    //Put Data in Mailbox
	
	}
	CANSTMOB = 0;
	
	CANCDMOB |= (1<<CONMOB0); /*enable transmition mode*/
	Wait_for_Can_Receive_message_Finished();	
	
}




 int8_t UART0_Init (void){
	 uint8_t intstate1 = SREG;/*save global interrupt flag*/
	 cli();/*disable interrupt*/
	 while(0<<TXC0){
	 }
	 /* set baud rate*/
	 UBRR0H = (uint8_t)(BAUD>>8);
	 UBRR0L = (uint8_t)(BAUD);
	 /* set frame format: i wil set it now as 8 bits, no parity 1 stop bit, asynchron*/
	 UCSR0C = (0<<UMSEL0)|(0<<UPM0)|(0<<USBS0)|(3<<UCSZ0);
	 /*set double speed*/
	 UCSR0A =(1<<U2X0);
	 /*enable receiver and transmitter; enable RX and Data register empty interrupt*/
	 UCSR0B = (1<<TXEN0)|(1<<RXEN0)|(1<<RXCIE0);
	 SREG=intstate1; /*restore global interrupt flag*/
	 return 1;
 }
  
/*
 *this function initializes the CAN register for AT90CAN128 and enable the CAN-controller */

 int8_t CAN_Init(void)

 {
	 uint8_t intstate2 = SREG;/*save global interrupt flag*/
	 cli();/*disable interrupt*/
	 CANGCON |= (1<<SWRES);                       
	 CANGCON &= ~(1<<SWRES); 
	 /*enable general interrupt interrupt*/
	 CANGIE= (1<<ENIT) | (1<<ENBOFF) |(1<<ENRX) |(1<<ENTX) |(1<<ENERR)|(1<<ENBX)|(1<<ENERG)|(0<<ENOVRT); 
	 /* enable Interrupt of MOBs register*/
	 CANIE2 = 0xFF;  
	 CANIE1 = 0x7F;  	
	
	 for (uint8_t mob = 0; mob < 15; mob++)       
	 {	
		 CANPAGE  = (mob << MOBNB0); /* clear all mailbox*/
		 CANSTMOB = 0x00;  /*clear  MOB status register*/
		 CANCDMOB = 0x00;  /*clear MOB control*/
		 CANGSTA = 0x00;   /*clear CAN general status register*/
		 /*clear all identifier tag register*/
		 CANIDT1=0X00;	
		 CANIDT2=0X00;
		 CANIDT3=0X00;
		 CANIDT4=0X00;
		 /*clear all identifier mask register*/
		 CANIDM1=0X00;	
		 CANIDM2=0X00;
		 CANIDM3=0X00;
		 CANIDM4=0X00;
		 /*clear all CAN message register*/
		 for (unsigned char j=0; j<8; j++) 
		 {	
			 CANMSG = 0;	        
		 }
	 }
	 /* set CAN baudrate register */
	
		 CANBT1 = (3<<BRP0);                                
		 CANBT2 = (4<<PRS0) | (0<<SJW0);                    
		 CANBT3 = (1<<PHS20) | (1<<PHS10);
		 
		 CANGCON |= (1<<ENASTB); /* set ENA/STB enable mode  */ 
		 CANCDMOB |= (0<<CONMOB0)|(0<<CONMOB1); /*enable reception mode here so that information for CAN come automatically*/  
		 SREG=intstate2; /*restore global interrupt flag*/
		 return 1;
	 
 }

 /*
 *this function deletes some CAN register for AT90CAN128 

 */
 void Init(void)
 {	
	 for (uint8_t mob = 0; mob < 15; mob++)         
	 {	
		 CANPAGE  = (mob << MOBNB0); /* clear all  mailbox*/
		 CANSTMOB = 0x00;  /*clear  MOB status register*/
		 CANCDMOB = 0x00;  /*clear MOB control register*/
		 CANGSTA = 0x00;   /*clear CAN general status register*/
		 /*clear all identifier tag register*/
		 CANIDT1=0X00;	
		 CANIDT2=0X00;
		 CANIDT3=0X00;
		 CANIDT4=0X00;
		 /*clear all identifier mask register*/
		 CANIDM1=0X00;	
		 CANIDM2=0X00;
		 CANIDM3=0X00;
		 CANIDM4=0X00;
		 for (unsigned char j=0; j<8; j++) 
		 {	
			 CANMSG = 0; /*clear all CAN message register*/
		 }
	 }
 }
 
 
  /*
 *this function deletes some variable
  */

 vod Reset_Parameter_CANReceive(void)
 {
	 /* all parameter initialisieren */
	 memset(ptr_canStruct, 0, sizeof(struct canStruct));
	
 }
 
 
 /*
 *this function wait until the command in case the request is send
 */

 void Wait_for_Can_Receive_message_Finished(void)
 {   
	  
	 double can_timeout2 = TIMEOUT_C; /*Timeout for CAN-communication*/
	 while (!( CANSTMOB & (1<<RXOK)) && (--can_timeout2>0)) 
	 {
		
		 /* do nothing  */
		  
	 }
	
     
	
	 CANSTMOB &= ~(1<<TXOK);  /* reset transmition flag */	
	 CANCDMOB &= ~(1<<CONMOB0); /* disable transmition mode */
	 
	
	
 }

 /*
 *This function grabs the CAN data received in a string
 the function has a poimter of the CAN structur as input and returns no parameter
 */
 
 void Convert_pack_canFrame_to_UartFormat(struct canStruct *ptr_canStruct)
 {
	 
	 /*all parameter initialisieren*/
	 Reset_Parameter_CANReceive();
	 
 }


 

 /* interrupt for receive data via CAN communication with defined interrupt vector CANIT_vect */ 

 ISR(CANIT_vect)
 {
	 uint8_t save_canpage = CANPAGE;  
	 mob = Get_Interrupt_of_Mob(CANSIT2 | (CANSIT1 << 8)); /*get the index of the MOb for which the interrupt occured*/
	 CANPAGE = mob<<4; /* select concerned MOb page */
	 if (CANSTMOB & (1<<TXOK))
	 {       CANSTMOB &= ~(1<<TXOK); /* disable transmition mode*/	  
	 CANCDMOB = (1<<CONMOB0); /* reset receive mode*/     
      
	 }

	 else if (CANSTMOB & (1<<RXOK))
	 {
		 ptr_canStruct->Can_Length = (CANCDMOB & (0xF<<DLC0));
		 for ( uint8_t i=0; i< ptr_canStruct->Can_Length; i++)
		 {	 /* get data of selected MOb */
			 ptr_canStruct->Can_Data[i]= CANMSG ;                
		 }
		 /*get identifier */
		 ptr_canStruct->Can_Message_ID = 0;
		 ptr_canStruct->Can_Message_ID = CANIDT2>>5;
		 ptr_canStruct->Can_Message_ID |= CANIDT1<<3; 
		 ptr_canStruct->Can_Mob = mob; /*get mailbox */
		 CANSTMOB &= ~(1<<RXOK); /* clear interrupt */	
		 CANCDMOB = (1<<CONMOB1); /* reset receive mode*/
		 canReady = 1; /* mark, that we got an CAN_interrupt, to be handled by main */
	 }
	 CANPAGE =  save_canpage;   /* restore CANPAGE*/
 }
 
 /* this function gets index of MOb for which an interrupt was detected  */


 	
 uint8_t Get_Interrupt_of_Mob( uint32_t bitmask) 
 {
	
	 if( bitmask == 0) return NOMOB;

 /*the bitmask is shifted to the rigth until the LSB is one
	 the number of such shift operations is the index of the MOb*/
	 for( mob = 0; (bitmask & 0x01) == 0; bitmask >>= 1, mob++);

	 /*security check*/
	 if ( mob > 14 ) return NOMOB;
	 else return mob;
 }

 
  